"""
Defines typedict to translate c types to numpy types

"""


typedict = {
    'float': 'f4',
    'double': 'f8',
    'char': 'i1',
    'short': 'i2',
    'long': 'i4',
    'long long': 'i8',
    'unsigned char': 'u1',
    'unsigned short': 'u2',
    'unsigned long': 'u4',
    'unsigned long long': 'u8',
    'float32_t': 'f4',
    'float64_t': 'f8',
    'int8_t': 'i1',
    'int16_t': 'i2',
    'int32_t': 'i4',
    'int64_t': 'i8',
    'uint8_t': 'u1',
    'uint16_t': 'u2',
    'uint32_t': 'u4',
    'uint64_t': 'u8',
    'long32': 'i4',
    'ulong32': 'u4',
    'int': 'i4',
    'unsigned int': 'u4',
}



# -*- coding: utf-8 -*-
"""
Created on Fri Jun 10 21:00:21 2016

@author: MaxBohnet
"""
from setuptools.dist import Distribution
from setuptools.command.build_ext import build_ext
import importlib.util
import numpy as np
import os
from cythonarrays.make_cython_extensions import make_extensions


class TestMakeExtensions:
    """Test making cython extensions"""

    def test_01_make_extension(self, tmpdir: str):
        """Test the creation of an extension"""
        source_dir = os.path.dirname(os.path.dirname(os.path.dirname(__file__)))
        dest_fn = tmpdir.strpath
        packagename = 'cythonarrays.tests.examplepackage'
        extension_name = f'{packagename}.example2_cython'
        # make the extension module
        extension_modules = make_extensions([extension_name],
                                            source_dir=source_dir)
        # test, if the extension module build in the fixture has the right name
        extension = extension_modules[0]
        assert extension.name == extension_name
        # and the according C-file
        assert extension.sources[0].split(os.sep)[-1] == 'example2_cython.c'

        # Create a distribution named examplepackage, including the extension modules
        dist = Distribution(dict(ext_modules=extension_modules))#
        dist.metadata.name = packagename
        dist.packages = [packagename]
        #  change the working directory to the test file's directory
        # to ensure the test runs from whereever it is called
        os.chdir(os.path.dirname(os.path.dirname(os.path.dirname(__file__))))
        package_dir = '.'
        dist.package_dir = {'': package_dir,}
        cmd = build_ext(dist)
        cmd.finalize_options()
        cmd.build_lib = dest_fn
        cmd.build_temp = dest_fn
        cmd.run()

        ext_fn = cmd.get_ext_filename(extension.name)
        file_location = os.path.join(dest_fn, ext_fn)
        # import the module from the file
        spec = importlib.util.spec_from_file_location(extension.name, file_location)
        example_cython = importlib.util.module_from_spec(spec)
        spec.loader.exec_module(example_cython)
        print('Import successful')

        # run a Cdef-function built with Openmp
        # and check if the results are as expected
        ex = example_cython.Example(2, 3, 4, 5)
        ret = ex.calc_array(22.2)
        np.testing.assert_almost_equal(ex.rowsums, [51.6, 251.4])
        assert ret == 151.5
        print('Calculation successful')



# -*- coding: utf-8 -*-
#cython: boundscheck=False
#cython: wraparound=False
#cython: cdivision=True
#cython: embedsignature=True


from cythonarrays.numpy_types cimport *
from cythonarrays.array_shapes cimport ArrayShapes



cdef class _Simple(ArrayShapes):
    """
    Cython CDefClass for Example model
    with coordinates groups and zones
    """





# -*- coding: utf-8 -*-

from cythonarrays.numpy_types cimport *
from cythonarrays.array_shapes cimport ArrayShapes
from cythonarrays.array_shapes import ArrayShapes


cdef class _Simple(ArrayShapes):
    """
    Cython CDefClass for Simple model
    with coordinates groups and zones
    """
    cdef public long32 groups
    cdef public long32 zones

    # zonenumbers
    cdef public ARRAY_1D_i4 _zonenumbers_i

    # parameter of groups
    cdef public ARRAY_1D_d _param_g

    # persons per group and zone
    cdef public ARRAY_2D_d _persons_gi




# -*- coding: utf-8 -*-

cimport cython


cdef class Example():
    """
    Cython CDefClass for Example class
    """
    cdef public cython.char n_threads
    cdef public cython.longlong large_value
    cdef public cython.double[:, :] array_ij
    cdef public cython.double[:] rowsums

    cpdef double calc_array(self, double param)





# -*- coding: utf-8 -*-


import numpy as np
np.seterr(divide='ignore', invalid='ignore')

from cythonarrays.array_properties import _ArrayProperties
import pyximport; pyximport.install()
from .simple_cython import (_Simple)



class Simple(_Simple, _ArrayProperties):
    """
    Simple job choice model

    Parameters
    ----------
    groups : int
        the number of groups
    zones : int
        the number of zones
    init_arrays: bool, optional(default=True)
        if true, the arrays are initialized in the beginning

    """
    _coordinates = {'zones': 'zonenumbers_i',
                    'groups': 'groupnames_g',
                    }

    def __init__(self,
                 groups,
                 zones,
                 init_arrays=True):
        super().__init__()

        self.groups = groups
        self.zones = zones

        self.define_arrays()
        if init_arrays:
            self.init_arrays()

    def define_arrays(self):
        """Define the arrays"""
        self.init_array('param_g', 'groups', -0.1)

        self.init_array('persons_gi', 'groups, zones')
        self.init_array('zonenumbers_i', 'zones')
        self.init_object_array('groupnames_g', 'groups')





# -*- coding: utf-8 -*-
#cython: boundscheck=False
#cython: wraparound=False
#cython: cdivision=True
#cython: embedsignature=True

cimport cython
import numpy as np

from cython.parallel import prange, threadid, parallel


cdef class Example:
    """
    Cython CDefClass for Example
    """
    def __init__(self, int i, int j, char n_threads, cython.longlong large_value):
        """
        Create the array with dimensions i and j

        Parameters
        ----------
        i:
            number of rows
        j:
            number of columns
        n_threads:
            number of threads
        large_value:
            a large value
        """
        self.array_ij = np.arange(i*j).astype('d').reshape((i, j))
        self.n_threads = n_threads
        self.large_value = large_value

    cpdef double calc_array(self, double param):
        """
        Calc an array with the given parameter

        Parameters
        ----------
        param:
            a parameter

        Returns
        -------
        :
            the result
        """
        cdef int i, j, t, n_rows, n_cols
        n_rows = self.array_ij.shape[0]
        n_cols = self.array_ij.shape[1]

        self.rowsums = np.zeros((n_rows))
        with nogil, parallel(num_threads=self.n_threads):
            t = threadid()
            # loop over calibration groups
            for i in prange(n_rows, schedule='guided'):
                for j in range(n_cols):
                    self.rowsums[i] += self.array_ij[i, j] * param - self.large_value
        return np.array(self.rowsums).mean()








from cythonarrays.configure_logger import get_logger


def function_with_logging(msg: str):
    """a function that logs something"""
    logger = get_logger(function_with_logging)
    logger.info(msg)